# encoding.py -- Functions to handle generalized encoding tasks...

import itertools
import math
import struct
import sys

def meta_encode(data):
    """encodes the data through many individual calls """

    # Currently assume these are fixed, but later we will allow them to changeg
    base = 256
    length = 6

    message = []
    remain_data = data
    while len(remain_data) > 0:
        data_chunk_bit = int(math.ceil(math.log(base**length, 2)))
        data_chunk_byte = data_chunk_bit / 8
        print data_chunk_byte

        chunk = remain_data[:data_chunk_byte] # Assume its text for now
        if len(chunk) < data_chunk_byte:
            chunk = chunk + (" " * (data_chunk_byte - len(chunk)))

        #print "Chunk:", len(chunk)

        choices = encode(chunk, base, length)
        message.append(choices)
        remain_data = remain_data[data_chunk_byte:]

    return message

def encode(data, base, length):
    """ Given a chunk, pick the stuff """
    # First compute the val of data
    val = 0

    for index, byte in enumerate(data):
        byte_val = ord(byte)
        val += byte_val * ((2**8)**index)

    # Ok lets just do this by hand I guess
    res = []
    r = val % base
    res.append(int(r))
    #q = math.floor(val / base)
    q=val / base
    while q:
        r = q % base
        #q = math.floor(q / base)
        q = q / base
        res.append(int(r))

    return res

def meta_decode(data):
    """same as the meta encode """

    base = 256
    length = 6

    total_string = ""
    for chunk in data:
        total_string += decode(chunk, length, base)

    return total_string

def decode(data, length, base):
    # Ok let's take that list and compute a val
    val = 0
    for index, place in enumerate(data):
        val += place *((base) ** index)

    # Now this part is the same as encode with base 10
    decode_base = 256 # Ie they are 8 bits
    res = []
    r = val % decode_base
    res.append(int(r))
    #q = math.floor(val / decode_base)
    q = val / decode_base
    while q:
        r = q % decode_base
        #q = math.floor(q/decode_base)
        q = q / decode_base
        res.append(int(r))

    #print len(res)
    # Ok, so we have a string of hex. We could have done this
    # from the get-go but this was easier... somehow
    #num_bytes = len(res)/2
    #print num_bytes
    #res_val = []
    #for byte_ind in range(0, num_bytes*2, 2):
    #    b_data = res[byte_ind:byte_ind+2]
    #    b_val = b_data[0] + decode_base * b_data[1]
    #    res_val.append(b_val)
    #print res_val
    res_val = res

    # Now take those and turn them back into strings
    char_array = [chr(x) for x in res_val]
    string_msg = "".join(char_array)

    return string_msg

def encode_chunk(data, n_choices, set_size):
    """ Attempts to encode the streaming data with the options given

    Returns a new list of the unencoded data. The idea is to call
    it repeatedly like a C write()
    """

    print set_size, n_choices

    # How much data can we fit in here?
    data_chunk_bit = int(math.floor(math.log(set_size**n_choices, 2)))
    data_chunk_byte = data_chunk_bit / 8
    print data_chunk_bit, data_chunk_byte


    # Get the actual data chunk
    chunk  = list(data[:data_chunk_byte])
    if len(chunk) < data_chunk_byte:
        chunk = chunk + (['\0'] * (data_chunk_byte - len(chunk)))

    # Encode
    choices = encode(chunk, set_size, n_choices)

    # What's left?
    remain_data = data[data_chunk_byte:]

    return choices, remain_data

if __name__ == "__main__":
    test_string = "Hello world"
    #print test_string
    #raw_data = meta_encode(test_string)
    #print raw_data
    #returned_string = meta_decode(raw_data)
    #print returned_string
    remain = test_string

    choice_list = []

    while len(remain) > 0:
        choices, remain = encode_chunk(remain, 5, 100)
        print choices
        choice_list.append(choices)

    print "choice list", choice_list
    # Let's decode it now:
    string = ""
    for choice in choice_list:
        string += decode(choice, 5, 100)

    print "FINAL OUTPUT: %s"%(string)
    print len(test_string), len(string)



